const textCacheQueryPlugin = (manager) => {
  const log = manager.createLogger("server.api.database.textCache");
  const context = manager.getContext();
  const { eventbus } = context;
  let cache = /* @__PURE__ */ new WeakMap();
  eventbus?.on("server", (event) => {
    if (event?.action == "databaseReloaded") {
      cache = /* @__PURE__ */ new WeakMap();
      log.trace("Clear query text cache");
    }
  });
  eventbus?.on("database", (event) => {
    if (event?.action == "updateEntries") {
      const entries = event?.entries || [];
      const cachedEntries = entries.filter((entry) => cache.has(entry));
      if (!cachedEntries.length) {
        return;
      }
      cachedEntries.forEach((entry) => cache.delete(entry));
      log.trace(`Cleared query text cache for ${cachedEntries.length} entries`);
    }
  });
  const cacheTextFn = (textFn) => {
    return (entry) => {
      if (!cache.has(entry)) {
        cache.set(entry, textFn(entry));
      }
      return cache.get(entry);
    };
  };
  return {
    name: "textCacheQueryPlugin",
    order: 100,
    transformRules: [
      {
        transform(ast, context2) {
          if (ast.type == "query") {
            const origTextFn = context2.textFn;
            context2.textFn = cacheTextFn(origTextFn);
          }
        }
      }
    ]
  };
};
const queryTextCache = {
  name: "queryTextCache",
  version: "1.0.0",
  async initialize(manager) {
    const log = manager.createLogger("server.api.database.textCache");
    const context = manager.getContext();
    if (context.type == "serverContext" || context.type == "cliContext") {
      await manager.register("query", textCacheQueryPlugin(manager));
      log.debug(`Use text cache for queries`);
    } else {
      log.debug(`Skip query text cache for non server contexts`);
    }
  }
};
var queryTextCache_default = queryTextCache;
export {
  queryTextCache_default as default
};
//# sourceMappingURL=queryTextCache.js.map
